/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Certus SDK, Version 7.0                    */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

import java.net.InetAddress;
import java.net.UnknownHostException;

/* IP Configuration */
public class ANPacket11
{
	public static final int LENGTH = 30;
	
	public boolean permanent;
	public boolean dhcpMode;
	public InetAddress ipAddress;
	public InetAddress netmask;
	public InetAddress gateway;
	public InetAddress dnsServer;

	public ANPacket11(ANPacket packet)
	{
		permanent = packet.data[0] != 0;
		dhcpMode = packet.data[1] != 0;
		try
		{
			ipAddress = bytestoInetAddress(packet.data, 2);
			netmask = bytestoInetAddress(packet.data, 6);
			gateway = bytestoInetAddress(packet.data, 10);
			dnsServer = bytestoInetAddress(packet.data, 14);
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
	}

	public ANPacket11()
	{
		permanent = true;
		dhcpMode = true;
		try
		{
			ipAddress = InetAddress.getByName("0.0.0.0");
			netmask = InetAddress.getByName("0.0.0.0");
			gateway = InetAddress.getByName("0.0.0.0");
			dnsServer = InetAddress.getByName("0.0.0.0");
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
	}

	public ANPacket toANPacket()
	{
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_NETWORK_CONFIGURATION);
		packet.data[0] = (byte) (permanent ? 1 : 0);
		packet.data[1] = (byte) (dhcpMode ? 1 : 0);
		iNetAddresstoBytes(ipAddress, packet.data, 2);
		iNetAddresstoBytes(netmask, packet.data, 6);
		iNetAddresstoBytes(gateway, packet.data, 10);
		iNetAddresstoBytes(dnsServer, packet.data, 14);
		for(int i = 18; i < 30; i++)
		{
			packet.data[i] = 0;
		}
		return packet;
	}

	public static InetAddress bytestoInetAddress(byte[] data, int index) throws UnknownHostException
	{
		byte[] address;
		address = new byte[]{ data[index + 3], data[index + 2], data[index + 1], data[index] };
		return InetAddress.getByAddress(address);
	}

	public static void iNetAddresstoBytes(InetAddress address, byte[] data, int index)
	{
		byte[] inverseOrder = address.getAddress();
		for (int i = 0; i < 4; i++)
		{
			data[index + i] = inverseOrder[3 - i];
		}
	}
}
