/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Certus SDK, Version 7.0                    */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Baud Rates */
public class ANPacket182
{
	public static final int LENGTH = 17;

	public static final int RS232_MODE = 0;
	public static final int RS422_MODE = 1;
	public boolean permanent;
	public boolean primary232Mode;
	@SuppressWarnings("unused")
	private boolean gpio1And2_232Mode;
	public boolean auxiliary232Mode;
	public long primaryBaudRate;
	public long gpio1And2BaudRate;
	public long auxiliaryBaudRate;

	public ANPacket182()
	{
	}

	public ANPacket182(ANPacket packet)
	{
		permanent = packet.data[0] != 0;

		long primaryBaudRaw = TypeConversion.bytesToUInt32(packet.data, 1);
		long gpio12BaudRaw = TypeConversion.bytesToUInt32(packet.data, 5);
		long auxiliaryBaudRaw = TypeConversion.bytesToUInt32(packet.data, 9);

		final long BAUD_MASK = (1L << 24) - 1L;
		final long MODE_MASK = (1L << 31);

		primaryBaudRate		= primaryBaudRaw	& BAUD_MASK;
		gpio1And2BaudRate	= gpio12BaudRaw		& BAUD_MASK;
		auxiliaryBaudRate	= auxiliaryBaudRaw	& BAUD_MASK;
		primary232Mode		= ((primaryBaudRaw	& MODE_MASK) == RS232_MODE);
		gpio1And2_232Mode	= ((gpio12BaudRaw	& MODE_MASK) == RS232_MODE);
		auxiliary232Mode	= ((auxiliaryBaudRaw & MODE_MASK) == RS232_MODE);
	}

	public ANPacket toANPacket() throws IndexOutOfBoundsException
	{
		ANPacket packet = new ANPacket(LENGTH, ANPacket.PACKET_ID_BAUD_RATE);

		packet.data[0] = (byte) (permanent ? 1 : 0);

		if (primaryBaudRate < 2400 || primaryBaudRate > 2000000) throw new IndexOutOfBoundsException();
		TypeConversion.int32ToBytes(packet.data, 1, (int) primaryBaudRate);
		if (!primary232Mode)
		{
			packet.data[4] = (byte)(1 << 7); // set msb of field #2 byte #4 if rs422 mode
		}

		if (gpio1And2BaudRate < 2400 || gpio1And2BaudRate > 250000) throw new IndexOutOfBoundsException();
		TypeConversion.int32ToBytes(packet.data, 5, (int) gpio1And2BaudRate);

		if (auxiliaryBaudRate < 2400 || auxiliaryBaudRate > 2000000) throw new IndexOutOfBoundsException();
		TypeConversion.int32ToBytes(packet.data, 9, (int) auxiliaryBaudRate);

		if (!auxiliary232Mode)
		{
			packet.data[12]	= (byte)(1 << 7); // set msb of field #4 byte #4 if rs422 mode
		}

		for (int i = 13; i < LENGTH; i++)
		{
			packet.data[i] = 0;
		}

		return packet;
	}

	public String modeString(boolean rs232mode)
	{
		if (rs232mode)
			return "RS-232";
		else
			return "RS-422";
	}
}
